#include "mainwindow.h"

#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qfile.h>
#include <q3filedialog.h>
#include <qcolor.h>
#include <qcolordialog.h>
#include <qbitmap.h>
#include <qinputdialog.h>
//Added by qt3to4:
#include <Q3StrList>
#include <QPixmap>
#include <QCloseEvent>
#include <QShowEvent>
#include <QHideEvent>
#include <Q3PopupMenu>
#include <Q3ActionGroup>
#include <Q3PtrList>
//#include <qdatetime.h>			// for random seed


#include <iostream>
#include <cstdlib>
#include <typeinfo>

#include "aboutdialog.h"
#include "exportoofiledialog.h"
#include "exports.h"
#include "exportxhtmldialog.h"
#include "file.h"
#include "flagrowobj.h"
#include "imports.h"
#include "mapeditor.h"
#include "misc.h"
#include "options.h"
#include "process.h"
#include "settings.h"
#include "showtextdialog.h"
#include "texteditor.h"
#include "version.h"

extern TextEditor *textEditor;
extern Main *mainWindow;
extern QString tmpVymDir;
extern QString clipboardDir;
extern bool clipboardEmpty;
extern int statusbarTime;
extern FlagRowObj* standardFlagsDefault;
extern FlagRowObj* systemFlagsDefault;

extern Q3PtrList <QAction> actionListBranches;

extern QAction* actionFileSave;
extern QAction* actionFilePrint;
extern QAction* actionEditUndo;
extern QAction* actionEditRedo;
extern QAction *actionEditCopy;
extern QAction *actionEditCut;
extern QAction *actionEditPaste;
extern QAction *actionEditMoveUp;
extern QAction *actionEditMoveDown;
extern QAction *actionEditToggleScroll;
extern QAction* actionEditOpenURL;
extern QAction* actionEditOpenURLTab;
extern QAction* actionEditURL;
extern QAction* actionEditHeading2URL;
extern QAction* actionEditBugzilla2URL;
extern QAction* actionEditFATE2URL;
extern QAction *actionEditOpenVymLink;
extern QAction *actionEditVymLink;
extern QAction *actionEditDeleteVymLink;
extern QAction *actionEditToggleHideExport;
extern QAction *actionEditMapInfo;
extern QAction *actionEditHeading;
extern QAction *actionEditDelete;
extern QAction *actionEditAddBranch;
extern QAction *actionEditAddBranchHere;
extern QAction *actionEditAddBranchAbove;
extern QAction *actionEditAddBranchBelow;
extern QAction *actionEditRemoveBranchKeepChilds;
extern QAction *actionEditRemoveChilds;
extern QAction *actionEditImportAdd;
extern QAction *actionEditImportReplace;
extern QAction *actionEditSaveBranch;
extern QAction *actionEditSelectFirst;
extern QAction *actionEditSelectLast;
extern QAction *actionEditLoadImage;

extern QAction* actionFormatColor;
extern QAction* actionFormatPickColor;
extern QAction* actionFormatColorBranch;
extern QAction* actionFormatColorSubtree;
extern QAction* actionFormatLinkColorHint;
extern QAction* actionFormatBackColor;
extern QAction* actionFormatLinkColor;
extern QAction *actionFormatIncludeImagesVer;
extern QAction *actionFormatIncludeImagesHor;

extern Q3ActionGroup* actionGroupModModes;
extern QAction* actionModModeColor;
extern QAction* actionModModeLink;
extern QAction* actionModModeCopy;

extern Q3ActionGroup *actionGroupFormatFrameTypes;
extern QAction *actionFormatFrameNone;
extern QAction *actionFormatFrameRectangle;

extern Q3ActionGroup *actionGroupFormatLinkStyles;
extern QAction *actionFormatLinkStyleLine;
extern QAction *actionFormatLinkStyleParabel;
extern QAction *actionFormatLinkStylePolyLine;
extern QAction *actionFormatLinkStylePolyParabel;
extern QAction *actionFormatHideLinkUnselected;

extern QAction *actionViewToggleNoteEditor;
extern QAction *actionViewToggleHistoryWindow;

extern QAction* actionSettingsAutoedit;
extern QAction* actionSettingsAutoselectHeading;
extern QAction* actionSettingsAutoselectHeading;
extern QAction* actionSettingsAutoselectText;
extern QAction* actionSettingsPasteNewHeading;
extern QAction* actionSettingsUseDelKey;
extern QAction* actionSettingsUseFlagGroups;
extern QAction* actionSettingsUseHideExport;

extern Q3PopupMenu* branchContextMenu;
extern Q3PopupMenu* branchAddContextMenu;
extern Q3PopupMenu* branchRemoveContextMenu;
extern Q3PopupMenu* branchLinksContextMenu;
extern Q3PopupMenu* branchLinksContextMenuDup;
extern Q3PopupMenu* floatimageContextMenu;
extern Q3PopupMenu* saveImageFormatMenu;
extern Q3PopupMenu* canvasContextMenu;
extern Q3PopupMenu* lastMapsMenu;
extern Q3PopupMenu* importMenu;
extern Q3PopupMenu* exportMenu;
extern Q3PopupMenu* exportImageFormatMenu;


extern Settings settings;
extern Options options;
extern QDir vymBaseDir;
extern QString iconPath;
extern QString flagsPath;

Main::Main(QWidget* parent, const char* name, Qt::WFlags f) :
    Q3MainWindow(parent,name,f)
{
	mainWindow=this;

	setCaption ("VYM - View Your Mind");

	// Load window settings
	resize (settings.readNumEntry( "/vym/mainwindow/geometry/width", 800),
	        settings.readNumEntry( "/vym/mainwindow/geometry/height",600));
	move   (settings.readNumEntry( "/vym/mainwindow/geometry/posX", 100),
	        settings.readNumEntry( "/vym/mainwindow/geometry/posY", 100));


	// Create unique temporary directory
	tmpVymDir=makeUniqueDir ("/tmp/vym-XXXXXX");

	// Create direcctory for clipboard
	clipboardDir=tmpVymDir+"/clipboard";
	QDir d(clipboardDir);
	d.mkdir (clipboardDir,true);
	makeSubDirs (clipboardDir);
	clipboardEmpty=true;

	procBrowser=NULL;

	// FIXME not used currently
	// Set random seed (random used for object IDs)
    // QTime t = QTime::currentTime();		// set random seed
    // srand( t.hour()*12+t.minute()*60+t.second()*60 );


	// Initialize some settings, which are platform dependant
	QString p,s;

		// application to open URLs
		p="/vym/mainwindow/readerURL";
		#if defined(Q_OS_LINUX)
			s=settings.readEntry (p,"konqueror");
		#else
			#if defined(Q_OS_MACX)
				s=settings.readEntry (p,"/usr/bin/open");
			#else
				s=settings.readEntry (p,"mozilla");
			#endif
		#endif
		settings.writeEntry( p,s);

		// application to open PDFs
		p="/vym/mainwindow/readerPDF";
		#if defined(Q_OS_LINUX)
			s=settings.readEntry (p,"acroread");
		#else
			#if defined(Q_OS_MACX)
				s=settings.readEntry (p,"/usr/bin/open");
			#else
				s=settings.readEntry (p,"acroread");
			#endif
		#endif
		settings.writeEntry( p,s);

	
	maxLastMaps=9;

	// Create tab widget which holds the maps
	tabWidget= new QTabWidget (this);
	connect( tabWidget, SIGNAL( currentChanged( QWidget * ) ), 
		this, SLOT( editorChanged( QWidget * ) ) );

	setCentralWidget(tabWidget);	

    setupFileActions();
    setupEditActions();
    setupFormatActions();
    setupViewActions();
    setupModeActions();
	setupFlagActions();
    setupSettingsActions();
	setupContextMenus();
    if (settings.readBoolEntry( "/vym/mainwindow/showTestMenu",false)) setupTestActions();
    setupHelpActions();
    
	// After menu is created, we can enable some actions
	actionFilePrint->setEnabled (true);

    statusBar();


	// Initialize Find window
	findWindow=new FindWindow(NULL,"findwindow");
	connect (findWindow, SIGNAL( findButton(QString) ), 
		this, SLOT(editFind(QString) ) );	
	connect (findWindow, SIGNAL( somethingChanged() ), 
		this, SLOT(editFindChanged() ) );	

	// Connect TextEditor, so that we can update flags if text changes
	connect (textEditor, SIGNAL (textHasChanged() ), this, SLOT (updateNoteFlag()));

	updateGeometry();
}

Main::~Main()
{
	// Save Settings
	settings.writeEntry( "/vym/mainwindow/geometry/width", width() );
	settings.writeEntry( "/vym/mainwindow/geometry/height", height() );
	settings.writeEntry( "/vym/mainwindow/geometry/posX", pos().x() );
	settings.writeEntry( "/vym/mainwindow/geometry/posY", pos().y() );

	settings.writeEntry( "/vym/version/version", __VYM_VERSION );
	settings.writeEntry( "/vym/version/builddate", __BUILD_DATE );

	settings.writeEntry( "/vym/mapeditor/editmode/autoselectheading",actionSettingsAutoselectHeading->isOn() );
	settings.writeEntry( "/vym/mapeditor/editmode/autoselecttext",actionSettingsAutoselectText->isOn() );
	settings.writeEntry( "/vym/mapeditor/editmode/pastenewheading",actionSettingsPasteNewHeading->isOn() );
	settings.writeEntry( "/vym/mapeditor/editmode/autoedit",actionSettingsAutoedit->isOn() );
	settings.writeEntry( "/vym/mapeditor/editmode/useDelKey",actionSettingsUseDelKey->isOn() );
	settings.writeEntry( "/vym/mapeditor/editmode/useFlagGroups",actionSettingsUseFlagGroups->isOn() );
	settings.writeEntry( "/vym/export/useHideExport",actionSettingsUseHideExport->isOn() );

	QString s;
	int maps=lastMaps.count();
	settings.writeEntry( "/vym/lastMaps/number",maps );
	for (int i=1;i<=maps;i++)
	{
		s=QString("/vym/lastMaps/map-%1").arg(i);
		if (!s.isEmpty() && i<=maxLastMaps) 
			settings.writeEntry (s, lastMaps.at(i-1));
	}


	// To make the texteditor save its settings, call the destructor
	delete (textEditor);

	// Remove temporary directory
	removeDir (QDir(tmpVymDir));
}

void Main::loadCmdLine()
{
	/* TODO draw some kind of splashscreen while loading...
	if (qApp->argc()>1)
	{
	}
	*/
	
	QStringList flist=options.getFileList();
	QStringList::Iterator it=flist.begin();

	while (it !=flist.end() )
	{
		fileLoad (*it, NewMap);
		*it++;
	}	
}


void Main::statusMessage(const QString &s)
{
	statusBar()->message (s);
}

void Main::closeEvent (QCloseEvent* )
{
	fileExitVYM();
}

// File Actions
void Main::setupFileActions()
{
    Q3ToolBar *tb = new Q3ToolBar( this );
    tb->setLabel( "File Actions" );
    Q3PopupMenu *menu = new Q3PopupMenu( this );
    menuBar()->insertItem( tr( "&File" ), menu );

	// Keycodes:  /usr/lib64/qt3/include/qnamespace.h

    QAction *a;
    a = new QAction(QPixmap( iconPath+"filenew.png"), tr( "&New..." ),this);
	a->setStatusTip ( tr( "New map","File menu" ) );
	a->setShortcut ( Qt::CTRL + Qt::Key_N );
    a->addTo( tb );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( fileNew() ) );
	
    a = new QAction( QPixmap( iconPath+"fileopen.png"), tr( "&Open..." ),this);
	a->setStatusTip (tr( "Open","File menu" ) );
	a->setShortcut ( Qt::CTRL + Qt::Key_O );
    a->addTo( tb );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( fileLoad() ) );
	
	lastMapsMenu = new Q3PopupMenu (this);

    menu->insertItem (tr("Open Recent"),lastMapsMenu );
    menu->insertSeparator();
	
    a = new QAction( QPixmap( iconPath+"filesave.png"), tr( "&Save..." ), this);
	a->setStatusTip ( tr( "Save" ));
	a->setShortcut (Qt::CTRL + Qt::Key_S );
    a->addTo( tb );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( fileSave() ) );
	actionFileSave=a;
	
    a = new QAction( QPixmap(iconPath+"filesaveas.png"), tr( "Save &As..." ), this);
	a->setStatusTip (tr( "Save &As" ) );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( fileSaveAs() ) );

    menu->insertSeparator();

	importMenu = new Q3PopupMenu (this);
    menu->insertItem (tr("Import"),importMenu );

	a = new QAction(tr("KDE Bookmarks"), this);
	a->setStatusTip ( tr( "Import")+" "+tr("KDE Bookmarks" ));
	a->addTo (importMenu);
	connect( a, SIGNAL( activated() ), this, SLOT( fileImportKDEBookmarks() ) );

    if (settings.readBoolEntry( "/vym/mainwindow/showTestMenu",false)) 
	{
		a = new QAction( QPixmap(), tr("Firefox Bookmarks"),this);
		a->setStatusTip (tr( "Import")+" "+tr("Firefox Bookmarks" ) );
		a->addTo (importMenu);
		connect( a, SIGNAL( activated() ), this, SLOT( fileImportFirefoxBookmarks() ) );
	}	

	a = new QAction("Mind Manager...",this);
	a->setStatusTip ( tr( "Import")+" Mind Manager"  );
	a->addTo (importMenu);
	connect( a, SIGNAL( activated() ), this, SLOT( fileImportMM() ) );

    a = new QAction( tr( "Import Dir"+QString("...") ), this);
	a->setStatusTip (tr( "Import directory structure (experimental)" ) );
    a->addTo( importMenu);
    connect( a, SIGNAL( activated() ), this, SLOT( fileImportDir() ) );

	exportMenu = new Q3PopupMenu (this);
    menu->insertItem (tr("Export"),exportMenu );

    menu->insertSeparator();


    a = new QAction(QPixmap( iconPath+"fileprint.png"), tr( "&Print")+QString("..."), this);
	a->setStatusTip ( tr( "Print" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_P );
    a->addTo( tb );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( filePrint() ) );
	actionFilePrint=a;

    a = new QAction( QPixmap(iconPath+"fileclose.png"), tr( "&Close Map" ), this);
	a->setStatusTip (tr( "Close Map" ) );
	a->setShortcut (Qt::ALT + Qt::Key_C );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( fileCloseMap() ) );

    a = new QAction(QPixmap(iconPath+"exit.png"), tr( "E&xit")+" "+__VYM, this);
	a->setStatusTip ( tr( "Exit")+" "+__VYM );
	a->setShortcut (Qt::CTRL + Qt::Key_Q );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( fileExitVYM() ) );
}


//Edit Actions
void Main::setupEditActions()
{
    Q3ToolBar *tb = new Q3ToolBar( this );
    tb->setLabel( "Edit Actions" );
    Q3PopupMenu *menu = new Q3PopupMenu( this );
    menuBar()->insertItem( tr( "&Edit" ), menu );

    QAction *a;
	QAction *alt;
    a = new QAction( QPixmap( iconPath+"undo.png"), tr( "&Undo" ),this);
    connect( a, SIGNAL( activated() ), this, SLOT( editUndo() ) );
	a->setStatusTip (tr( "Undo" ) );
	a->setShortcut ( Qt::CTRL + Qt::Key_Z );
	a->setEnabled (false);
    a->addTo( tb );
    a->addTo( menu );
	actionEditUndo=a;
    
    if (settings.readBoolEntry( "/vym/mainwindow/showTestMenu",false)) 
	{
		a = new QAction( QPixmap( iconPath+"redo.png"), tr( "&Redo" ), this); 
		a->setStatusTip (tr( "Redo" ));
		a->setShortcut (Qt::CTRL + Qt::Key_Y );
		a->addTo( tb );
		a->addTo( menu );
		connect( a, SIGNAL( activated() ), this, SLOT( editRedo() ) );
	}
   
    menu->insertSeparator();
    a = new QAction(QPixmap( iconPath+"editcopy.png"), tr( "&Copy" ), this);
	a->setStatusTip ( tr( "Copy" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_C );
	a->setEnabled (false);
    a->addTo( tb );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( editCopy() ) );
	actionEditCopy=a;
	
    a = new QAction(QPixmap( iconPath+"editcut.png" ), tr( "Cu&t" ), this);
	a->setStatusTip ( tr( "Cut" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_X );
	a->setEnabled (false);
    a->addTo( tb );
    a->addTo( menu );
	actionEditCut=a;
    connect( a, SIGNAL( activated() ), this, SLOT( editCut() ) );
	
    a = new QAction(QPixmap( iconPath+"editpaste.png"), tr( "&Paste" ),this);
    connect( a, SIGNAL( activated() ), this, SLOT( editPaste() ) );
	a->setStatusTip ( tr( "Paste" ) );
	a->setShortcut ( Qt::CTRL + Qt::Key_V );
	a->setEnabled (false);
    a->addTo( tb );
    a->addTo( menu );
	actionEditPaste=a;

    a = new QAction(QPixmap(iconPath+"up.png" ), tr( "Move up" ), this);
	a->setStatusTip ( tr( "Move branch up" ) );
	a->setShortcut (Qt::Key_PageUp );
	a->setEnabled (false);
    a->addTo( tb );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( editMoveUp() ) );
	actionEditMoveUp=a;

    a = new QAction( QPixmap( iconPath+"down.png"), tr( "Move down" ),this);
    connect( a, SIGNAL( activated() ), this, SLOT( editMoveDown() ) );
	a->setStatusTip (tr( "Move branch down" ) );
	a->setShortcut ( Qt::Key_PageDown );
	a->setEnabled (false);
    a->addTo( tb );
    a->addTo( menu );
	actionEditMoveDown=a;
	

	a = new QAction( QPixmap(flagsPath+"flag-scrolled-right.png"), tr( "Scroll branch" ),this);
	a->setShortcut ( Qt::Key_ScrollLock );
	a->setStatusTip (tr( "Scroll branch" ) );
    connect( a, SIGNAL( activated() ), this, SLOT( editToggleScroll() ) );
	alt = new QAction( QPixmap(flagsPath+"flag-scrolled-right.png"), tr( "Scroll branch" ), this);
	alt->setShortcut ( Qt::Key_S );
	alt->setStatusTip (tr( "Scroll branch" )); 
    connect( alt, SIGNAL( activated() ), this, SLOT( editToggleScroll() ) );
	#if defined(Q_OS_MACX)
		actionEditToggleScroll=alt;
	#else	
		actionEditToggleScroll=a;
	#endif	
	actionEditToggleScroll->setEnabled (false);
	actionEditToggleScroll->setToggleAction(true);
    actionEditToggleScroll->addTo( tb );
    actionEditToggleScroll->addTo( menu );
	actionListBranches.append(actionEditToggleScroll);
	
    a = new QAction( tr( "Unscroll all scrolled branches" ), this);
	a->setStatusTip (tr( "Unscroll all" ));
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( editUnScrollAll() ) );
	
    menu->insertSeparator();

	a = new QAction( QPixmap(iconPath+"find.png"), tr( "Find"+QString("...") ), this);
	a->setStatusTip (tr( "Find" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_F );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( editOpenFindWindow() ) );
    
	menu->insertSeparator();

	a = new QAction( QPixmap(flagsPath+"flag-url.png"), tr( "Open URL" ), this);
	a->setShortcut (Qt::CTRL + Qt::Key_U );
	a->setShortcut (tr( "Open URL" ));
    a->addTo( tb );
	a->setEnabled (false);
    connect( a, SIGNAL( activated() ), this, SLOT( editOpenURL() ) );
	actionEditOpenURL=a;

	a = new QAction( tr( "Open URL in new tab" ), this);
	a->setStatusTip (tr( "Open URL in new tab" ));
	a->setShortcut (Qt::CTRL+Qt::Key_U );
	a->setEnabled (false);
    connect( a, SIGNAL( activated() ), this, SLOT( editOpenURLTab() ) );
	actionEditOpenURLTab=a;

	a = new QAction(QPixmap(), tr( "Edit URL"+QString("...") ), this);
	a->setStatusTip ( tr( "Edit URL" ) );
	a->setShortcut (Qt::SHIFT + Qt::CTRL+Qt::Key_U );
	a->setEnabled (false);
	actionListBranches.append(a);
    connect( a, SIGNAL( activated() ), this, SLOT( editURL() ) );
	actionEditURL=a;
	
	a = new QAction( tr( "Use heading for URL" ), this);
	a->setStatusTip ( tr( "Use heading of selected branch as URL" ));
	a->setEnabled (false);
	actionListBranches.append(a);
    connect( a, SIGNAL( activated() ), this, SLOT( editHeading2URL() ) );
	actionEditHeading2URL=a;
    
	a = new QAction(tr( "Create URL to Bugzilla" ), this);
	a->setStatusTip ( tr( "Create URL to Bugzilla" ));
	a->setEnabled (false);
	actionListBranches.append(a);
    connect( a, SIGNAL( activated() ), this, SLOT( editBugzilla2URL() ) );
	actionEditBugzilla2URL=a;
    
	a = new QAction(tr( "Create URL to FATE" ), this);
	a->setStatusTip ( tr( "Create URL to FATE" ));
	a->setEnabled (false);
	actionListBranches.append(a);
    connect( a, SIGNAL( activated() ), this, SLOT( editFATE2URL() ) );
	actionEditFATE2URL=a;
	
    a = new QAction(QPixmap(flagsPath+"flag-vymlink.png"), tr( "Jump to map" ), this);
	a->setStatusTip ( tr( "Jump to another vym map, if needed load it first" ));
    a->addTo( tb );
	a->setEnabled (false);
    connect( a, SIGNAL( activated() ), this, SLOT( editOpenVymLink() ) );
	actionEditOpenVymLink=a;
	
    a = new QAction(tr( "Edit vym link"+QString("...") ), this);
	a->setEnabled (false);
	a->setStatusTip ( tr( "Edit link to another vym map" ));
    connect( a, SIGNAL( activated() ), this, SLOT( editVymLink() ) );
	actionListBranches.append(a);
	actionEditVymLink=a;

    a = new QAction(tr( "Delete vym link" ),this);
	a->setStatusTip ( tr( "Delete link to another vym map" ));
	a->setEnabled (false);
    connect( a, SIGNAL( activated() ), this, SLOT( editDeleteVymLink() ) );
	actionEditDeleteVymLink=a;

    a = new QAction(QPixmap(flagsPath+"flag-hideexport.png"), tr( "Hide in exports" ), this);
	a->setStatusTip ( tr( "Hide object in exports" ) );
	a->setShortcut (Qt::Key_H );
	a->setToggleAction(true);
    a->addTo( tb );
	a->setEnabled (false);
    connect( a, SIGNAL( activated() ), this, SLOT( editToggleHideExport() ) );
	actionEditToggleHideExport=a;

    a = new QAction(tr( "Edit Map Info"+QString("...") ),this);
	a->setStatusTip ( tr( "Edit Map Info" ));
	a->setEnabled (true);
    connect( a, SIGNAL( activated() ), this, SLOT( editMapInfo() ) );
	actionEditMapInfo=a;

	menu->insertSeparator();

    // Shortcuts to modify heading:
    a = new QAction(tr( "Edit heading" ),this);
	a->setStatusTip ( tr( "edit Heading" ));
	a->setShortcut ( Qt::Key_Enter );
    connect( a, SIGNAL( activated() ), this, SLOT( editHeading() ) );
	actionListBranches.append(a);
    a = new QAction( tr( "Edit heading" ), this);
	a->setStatusTip (tr( "edit Heading" ));
	a->setShortcut (Qt::Key_Return );
    connect( a, SIGNAL( activated() ), this, SLOT( editHeading() ) );
	actionListBranches.append(a);
	actionEditHeading=a;
    a = new QAction( tr( "Edit heading" ),this);
	a->setStatusTip (tr( "edit Heading" ));
	a->setShortcut ( Qt::Key_F2 );
    connect( a, SIGNAL( activated() ), this, SLOT( editHeading() ) );
	a->setEnabled (false);
	actionEditHeading=a;
	actionListBranches.append(a);
    
    // Shortcut to delete selection
    a = new QAction( tr( "Delete Selection" ),this);
	a->setStatusTip (tr( "Delete Selection" ));
	a->setShortcut ( Qt::Key_Delete);
    connect( a, SIGNAL( activated() ), this, SLOT( editDeleteSelection() ) );
	a->setEnabled (false);
	actionEditDelete=a;
    
    // Shortcut to add branch
	alt = new QAction(tr( "Add branch as child" ), this);
	alt->setStatusTip ( tr( "Add a branch as child of selection" ));
	alt->setShortcut (Qt::Key_A);
    connect( alt, SIGNAL( activated() ), this, SLOT( editNewBranch() ) );
	a = new QAction(tr( "Add branch as child" ), this);
	a->setStatusTip ( tr( "Add a branch as child of selection" ));
	a->setShortcut (Qt::Key_Insert);	
	a->setEnabled (false);
    connect( a, SIGNAL( activated() ), this, SLOT( editNewBranch() ) );
	actionListBranches.append(a);
	#if defined (Q_OS_MACX)
		// In OSX show different shortcut in menues, the keys work independtly always			
		actionEditAddBranch=alt;
	#else	
		actionEditAddBranch=a;
	#endif	

    // Add branch by inserting it at selection
	a = new QAction(tr( "Add branch (insert)" ), this);
	a->setStatusTip ( tr( "Add a branch by inserting and making selection its child" ));
	a->setShortcut (Qt::ALT + Qt::Key_Insert );
    connect( a, SIGNAL( activated() ), this, SLOT( editNewBranchHere() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionEditAddBranchHere=a;
	a = new QAction(tr( "Add branch (insert)" ),this);
	a->setStatusTip ( tr( "Add a branch by inserting and making selection its child" ));
	a->setShortcut ( Qt::ALT + Qt::Key_A );
    connect( a, SIGNAL( activated() ), this, SLOT( editNewBranchHere() ) );
	actionListBranches.append(a);

	// Add branch above
    a = new QAction(tr( "Add branch above" ), this);
	a->setStatusTip ( tr( "Add a branch above selection" ));
	a->setShortcut (Qt::SHIFT+Qt::Key_Insert );
    connect( a, SIGNAL( activated() ), this, SLOT( editNewBranchAbove() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionEditAddBranchAbove=a;
    a = new QAction(tr( "Add branch above" ), this);
	a->setStatusTip ( tr( "Add a branch above selection" ));
	a->setShortcut (Qt::SHIFT+Qt::Key_A );
    connect( a, SIGNAL( activated() ), this, SLOT( editNewBranchAbove() ) );
	actionListBranches.append(a);

	// Add branch below 
    a = new QAction(tr( "Add branch below" ), this);
	a->setStatusTip ( tr( "Add a branch below selection" ));
	a->setShortcut (Qt::CTRL +Qt::Key_Insert );
    connect( a, SIGNAL( activated() ), this, SLOT( editNewBranchBelow() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionEditAddBranchBelow=a;
    a = new QAction(tr( "Add branch below" ), this);
	a->setStatusTip ( tr( "Add a branch below selection" ));
	a->setShortcut (Qt::CTRL +Qt::Key_A );
    connect( a, SIGNAL( activated() ), this, SLOT( editNewBranchBelow() ) );
	actionListBranches.append(a);

	// Import at selection (adding to selection)
    a = new QAction( tr( "Add map (insert)" ),this);
	a->setStatusTip (tr( "Add map at selection" ));
    connect( a, SIGNAL( activated() ), this, SLOT( editImportAdd() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionEditImportAdd=a;

	// Import at selection (replacing selection)
    a = new QAction( tr( "Add map (replace)" ), this);
	a->setStatusTip (tr( "Replace selection with map" ));
    connect( a, SIGNAL( activated() ), this, SLOT( editImportReplace() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionEditImportReplace=a;

	// Save selection 
    a = new QAction( tr( "Save selection" ), this);
	a->setStatusTip (tr( "Save selection" ));
    connect( a, SIGNAL( activated() ), this, SLOT( editSaveBranch() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionEditSaveBranch=a;

	// Only remove branch, not its childs
    a = new QAction(tr( "Remove only branch " ), this);
	a->setStatusTip ( tr( "Remove only branch and keep its childs" ));
	a->setShortcut (Qt::ALT + Qt::Key_Delete );
    connect( a, SIGNAL( activated() ), this, SLOT( editRemoveBranchKeepChilds() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionEditRemoveBranchKeepChilds=a;

	// Only remove childs of a branch
    a = new QAction( tr( "Remove childs" ), this);
	a->setStatusTip (tr( "Remove childs of branch" ));
	a->setShortcut (Qt::SHIFT + Qt::Key_Delete );
    connect( a, SIGNAL( activated() ), this, SLOT( editRemoveChilds() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionEditRemoveChilds=a;

    // Shortcuts for navigating with cursor:
    a = new QAction(tr( "Select upper branch" ), this);
	a->setStatusTip ( tr( "Select upper branch" ));
	a->setShortcut (Qt::Key_Up );
    connect( a, SIGNAL( activated() ), this, SLOT( editUpperBranch() ) );
    a = new QAction( tr( "Select lower branch" ),this);
	a->setStatusTip (tr( "Select lower branch" ));
	a->setShortcut ( Qt::Key_Down );
    connect( a, SIGNAL( activated() ), this, SLOT( editLowerBranch() ) );
    a = new QAction(tr( "Select left branch" ), this);
	a->setStatusTip ( tr( "Select left branch" ));
	a->setShortcut (Qt::Key_Left );
    connect( a, SIGNAL( activated() ), this, SLOT( editLeftBranch() ) );
    a = new QAction( tr( "Select child branch" ), this);
	a->setStatusTip (tr( "Select right branch" ));
	a->setShortcut (Qt::Key_Right);
    connect( a, SIGNAL( activated() ), this, SLOT( editRightBranch() ) );
    a = new QAction( tr( "Select first branch" ), this);
	a->setStatusTip (tr( "Select first branch" ));
	a->setShortcut (Qt::Key_Home );
	a->setEnabled (false);
    a->addTo ( menu );
	actionListBranches.append(a);
	actionEditSelectFirst=a;
    connect( a, SIGNAL( activated() ), this, SLOT( editFirstBranch() ) );
    a = new QAction( tr( "Select last branch" ),this);
	a->setStatusTip (tr( "Select last branch" ));
	a->setShortcut ( Qt::Key_End );
    connect( a, SIGNAL( activated() ), this, SLOT( editLastBranch() ) );
	a->setEnabled (false);
    a->addTo ( menu );
	actionListBranches.append(a);
	actionEditSelectLast=a;

    a = new QAction( tr( "Add Image" )+QString("..."), this);
	a->setStatusTip (tr( "Add Image" ));
    connect( a, SIGNAL( activated() ), this, SLOT( editLoadImage() ) );
	actionEditLoadImage=a;

}

// Format Actions
void Main::setupFormatActions()
{
    Q3PopupMenu *menu = new Q3PopupMenu( this );
    menuBar()->insertItem( tr( "F&ormat" ), menu );

    Q3ToolBar *tb = new Q3ToolBar( this );
    QAction *a;
    QPixmap pix( 16,16);
    pix.fill (Qt::black);
    a= new QAction(pix, tr( "Set &Color" )+QString("..."), this);
	a->setStatusTip ( tr( "Set Color" ));
    connect( a, SIGNAL( activated() ), this, SLOT( formatSelectColor() ) );
    a->addTo( tb );
    a->addTo( menu );
	actionFormatColor=a;
    a= new QAction( QPixmap(iconPath+"formatcolorpicker.png"), tr( "Pic&k color" ), this);
	a->setStatusTip (tr( "Pick color\nHint: You can pick a color from another branch and color using CTRL+Left Button" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_K );
    connect( a, SIGNAL( activated() ), this, SLOT( formatPickColor() ) );
	a->setEnabled (false);
    a->addTo( tb );
    a->addTo( menu );
	actionListBranches.append(a);
	actionFormatPickColor=a;

    a= new QAction(QPixmap(iconPath+"formatcoloritem.png"), tr( "Color &branch" ), this);
	a->setStatusTip ( tr( "Color branch" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_I);
    connect( a, SIGNAL( activated() ), this, SLOT( formatColorItem() ) );
	a->setEnabled (false);
    a->addTo( tb );
    a->addTo( menu );
	actionListBranches.append(a);
	actionFormatColorBranch=a;

    a= new QAction(QPixmap(iconPath+"formatcolorbranch.png"), tr( "Color sub&tree" ), this);
	a->setStatusTip ( tr( "Color Subtree" ));
	a->setShortcut (Qt::CTRL + Qt::Key_T);
    connect( a, SIGNAL( activated() ), this, SLOT( formatColorBranch() ) );
	a->setEnabled (false);
    a->addTo( menu );
    a->addTo( tb );
	actionListBranches.append(a);
	actionFormatColorSubtree=a;

    menu->insertSeparator();
	actionGroupFormatLinkStyles=new Q3ActionGroup ( this, "formatLinkStyles");
	actionGroupFormatLinkStyles->setExclusive (true);
    a= new QAction( tr( "Linkstyle Line" ), actionGroupFormatLinkStyles);
	a->setStatusTip (tr( "Line" ));
	a->setToggleAction(true);
    connect( a, SIGNAL( activated() ), this, SLOT( formatLinkStyleLine() ) );
	actionFormatLinkStyleLine=a;
    a= new QAction( tr( "Linkstyle Parabel" ), actionGroupFormatLinkStyles);
	a->setStatusTip (tr( "Line" ));
	a->setToggleAction(true);
    connect( a, SIGNAL( activated() ), this, SLOT( formatLinkStyleParabel() ) );
	actionFormatLinkStyleParabel=a;
    a= new QAction( tr( "Linkstyle Thick Line" ), actionGroupFormatLinkStyles );
	a->setStatusTip (tr( "PolyLine" ));
	a->setToggleAction(true);
    connect( a, SIGNAL( activated() ), this, SLOT( formatLinkStylePolyLine() ) );
	actionFormatLinkStylePolyLine=a;
    a= new QAction( tr( "Linkstyle Thick Parabel" ), actionGroupFormatLinkStyles);
	a->setStatusTip (tr( "PolyParabel" ) );
	a->setToggleAction(true);
    connect( a, SIGNAL( activated() ), this, SLOT( formatLinkStylePolyParabel() ) );
	actionFormatLinkStylePolyParabel=a;
	actionGroupFormatLinkStyles->addTo (menu);
	
	actionGroupFormatFrameTypes=new Q3ActionGroup ( this, "formatFrameTypes");
	actionGroupFormatFrameTypes->setExclusive (true);
    a = new QAction( tr( "No Frame" ), actionGroupFormatFrameTypes );
	a->setStatusTip (tr("No Frame"));
	a->setToggleAction(true);
    connect( a, SIGNAL( activated() ), this, SLOT( formatFrameNone() ) );
	actionFormatFrameNone=a;
    a = new QAction( tr( "Rectangle" ), actionGroupFormatFrameTypes);
	a->setStatusTip (tr( "Rectangle" ));
	a->setToggleAction(true);
    connect( a, SIGNAL( activated() ), this, SLOT( formatFrameRectangle() ) );
	actionFormatFrameRectangle=a;

    a = new QAction( tr( "Include images vertically" ),  actionFormatIncludeImagesVer);
	a->setStatusTip ( tr ("Include top and bottom position of images into branch"));
	a->setToggleAction(true);
    connect( a, SIGNAL( activated() ), this, SLOT( formatIncludeImagesVer() ) );
	actionFormatIncludeImagesVer=a;

    a = new QAction( tr( "Include images horizontally" ),  actionFormatIncludeImagesHor );
	a->setStatusTip ( tr ("Include left and right position of images into branch"));
	a->setToggleAction(true);
    connect( a, SIGNAL( activated() ), this, SLOT( formatIncludeImagesHor() ) );
	actionFormatIncludeImagesHor=a;

    a = new QAction( tr( "Hide link if object is not selected" ), actionFormatHideLinkUnselected);
	a->setStatusTip (tr( "Hide link" ));
	a->setToggleAction(true);
    connect( a, SIGNAL( activated() ), this, SLOT( formatHideLinkUnselected() ) );
	actionFormatHideLinkUnselected=a;

    menu->insertSeparator();
    a= new QAction( tr( "&Use color of heading for link" ),  this);
	a->setStatusTip (tr( "Use same color for links and headings" ));
	a->setToggleAction(true);
    connect( a, SIGNAL( activated() ), this, SLOT( formatToggleLinkColorHint() ) );
	a->addTo( menu );
	actionFormatLinkColorHint=a;

    pix.fill (Qt::white);
    a= new QAction( pix, tr( "Set &Link Color"+QString("...") ), this  );
	a->setStatusTip (tr( "Set Link Color" ));
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( formatSelectLinkColor() ) );
    actionFormatLinkColor=a;

    a= new QAction( pix, tr( "Set &Background Color" )+QString("..."), this );
	a->setStatusTip (tr( "Set Background Color" ));
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( formatSelectBackColor() ) );
    actionFormatBackColor=a;
}

// View Actions
void Main::setupViewActions()
{
    Q3ToolBar *tb = new Q3ToolBar( this );
    tb->setLabel( "View Actions" );
    Q3PopupMenu *menu = new Q3PopupMenu( this );
    menuBar()->insertItem( tr( "&View" ), menu );

    QAction *a;
    a = new QAction(QPixmap(iconPath+"viewmag-reset.png"), tr( "reset Zoom" ), this);
	a->setStatusTip ( tr( "Zoom reset" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_0 );
    a->addTo( tb );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT(viewZoomReset() ) );
	
    a = new QAction( QPixmap(iconPath+"viewmag+.png"), tr( "Zoom in" ), this);
	a->setStatusTip (tr( "Zoom in" ));
	a->setShortcut (Qt::CTRL + Qt::Key_Plus);
    a->addTo( tb );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT(viewZoomIn() ) );
	
    a = new QAction( QPixmap(iconPath+"viewmag-.png"), tr( "Zoom out" ), this);
	a->setStatusTip (tr( "Zoom out" ));
	a->setShortcut (Qt::CTRL + Qt::Key_Minus );
    a->addTo( tb );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( viewZoomOut() ) );


    a = new QAction(QPixmap(flagsPath+"flag-note.png"), tr( "Show Note Editor" ),this);
	a->setStatusTip ( tr( "Show Note Editor" ));
	a->setShortcut ( Qt::CTRL + Qt::Key_E );
	a->setToggleAction(true);
	if (textEditor->showWithMain())
		a->setOn(true);
	else	
		a->setOn(false);
    a->addTo( tb );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT(windowToggleNoteEditor() ) );
	actionViewToggleNoteEditor=a;

    a = new QAction(  tr( "Show history window" ),this );
	a->setStatusTip ( tr( "Show history window" ));
	a->setShortcut ( Qt::CTRL + Qt::Key_H  );
	a->setToggleAction(false);
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT(windowToggleHistory() ) );
	actionViewToggleHistoryWindow=a;

    a = new QAction(tr( "Next Window" ), this);
	a->setStatusTip ( tr( "&Next Window" ) );
	a->setShortcut (Qt::ALT + Qt::Key_N );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT(windowNextEditor() ) );

    a = new QAction (tr( "Previous Window" ), this );
	a->setStatusTip (tr( "&Previous Window" ));
	a->setShortcut (Qt::ALT + Qt::Key_P );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT(windowPreviousEditor() ) );
}

// Mode Actions
void Main::setupModeActions()
{
    //QPopupMenu *menu = new QPopupMenu( this );
    //menuBar()->insertItem( tr( "&Mode (using modifiers)" ), menu );

    Q3ToolBar *tb = new Q3ToolBar( this );
    tb->setLabel( tr ("Modes when using modifiers") );
    QAction *a;
	actionGroupModModes=new Q3ActionGroup ( this, "formatLinkStyles");
	actionGroupModModes->setExclusive (true);
    a= new QAction( QPixmap(iconPath+"modecolor.png"), tr( "Use modifier to color branches" ), actionGroupModModes);
	a->setShortcut (Qt::Key_J);
    a->setStatusTip ( tr( "Use modifier to color branches" ));
	a->setToggleAction(true);
	a->addTo (tb);
	a->setOn(true);
	actionModModeColor=a;
	
    a= new QAction( QPixmap(iconPath+"modecopy.png"), tr( "Use modifier to copy" ), actionGroupModModes );
	a->setShortcut( Qt::Key_K); 
    a->setStatusTip( tr( "Use modifier to copy" ));
	a->setToggleAction(true);
	a->addTo (tb);
	actionModModeCopy=a;

    a= new QAction(QPixmap(iconPath+"modelink.png"), tr( "Use modifier to draw xLinks" ), actionGroupModModes );
	a->setShortcut (Qt::Key_L);
    a->setStatusTip( tr( "Use modifier to draw xLinks" ));
	a->setToggleAction(true);
	a->addTo (tb);
	actionModModeLink=a;
}

// Flag Actions
void Main::setupFlagActions()
{
	// Create System Flags
	systemFlagsDefault = new FlagRowObj ();
	systemFlagsDefault->setVisibility (false);
	systemFlagsDefault->setName ("systemFlagsDef");

	FlagObj *fo = new FlagObj ();
	fo->load(QPixmap(flagsPath+"flag-note.png"));
	fo->setName("note");
	fo->setToolTip(tr("Note","Systemflag"));
	systemFlagsDefault->addFlag (fo);	// makes deep copy

	fo->load(QPixmap(flagsPath+"flag-url.png"));
	fo->setName("url");
	fo->setToolTip(tr("WWW Document (external)","Systemflag"));
	systemFlagsDefault->addFlag (fo);
	
	fo->load(QPixmap(flagsPath+"flag-vymlink.png"));
	fo->setName("vymLink");
	fo->setToolTip(tr("Link to another vym map","Systemflag"));
	systemFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-scrolled-right.png"));
	fo->setName("scrolledright");
	fo->setToolTip(tr("subtree is scrolled","Systemflag"));
	systemFlagsDefault->addFlag (fo);
	
	fo->load(QPixmap(flagsPath+"flag-tmpUnscrolled-right.png"));
	fo->setName("tmpUnscrolledright");
	fo->setToolTip(tr("subtree is temporary scrolled","Systemflag"));
	systemFlagsDefault->addFlag (fo);

	fo->load(QPixmap(flagsPath+"flag-hideexport.png"));
	fo->setName("hideInExport");
	fo->setToolTip(tr("Hide object in exported maps","Systemflag"));
	systemFlagsDefault->addFlag (fo);

	// Create Standard Flags
	standardFlagsDefault = new FlagRowObj ();
	standardFlagsDefault->setVisibility (false);
	standardFlagsDefault->setName ("standardFlagsDef");

	fo->load(QPixmap(flagsPath+"flag-exclamationmark.png"));
	fo->setName ("exclamationmark");
	fo->setGroup("standard-mark");
	fo->setToolTip(tr("Take care!","Standardflag"));
	standardFlagsDefault->addFlag (fo);	// makes deep copy
	
	fo->load(QPixmap(flagsPath+"flag-questionmark.png"));
	fo->setName("questionmark");
	fo->setGroup("standard-mark");
	fo->setToolTip(tr("Really?","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-hook-green.png"));
	fo->setName("hook-green");
	fo->setGroup("standard-hook");
	fo->setToolTip(tr("ok!","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-cross-red.png"));
	fo->setName("cross-red");
	fo->setGroup("standard-hook");
	fo->setToolTip(tr("Not ok!","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-stopsign.png"));
	fo->setName("stopsign");
	fo->setToolTip(tr("This won't work!","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-smiley-good.png"));
	fo->setName("smiley-good");
	fo->setGroup("standard-smiley");
	fo->setToolTip(tr("Good","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-smiley-sad.png"));
	fo->setName("smiley-sad");
	fo->setGroup("standard-smiley");
	fo->setToolTip(tr("Bad","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-smiley-omg.png"));
	// Original omg.png (in KDE emoticons)
	fo->setName("smiley-omg");
	fo->setGroup("standard-smiley");
	fo->setToolTip(tr("Oh no!","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-kalarm.png"));
	fo->setName("clock");
	fo->setToolTip(tr("Time critical","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-phone.png"));
	fo->setName("phone");
	fo->setToolTip(tr("Call...","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-lamp.png"));
	fo->setName("lamp");
	fo->setToolTip(tr("Idea!","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-arrow-up.png"));
	fo->setName("arrow-up");
	fo->setGroup("standard-arrow");
	fo->setToolTip(tr("Important","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-arrow-down.png"));
	fo->setName("arrow-down");
	fo->setGroup("standard-arrow");
	fo->setToolTip(tr("Unimportant","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-arrow-2up.png"));
	fo->setName("2arrow-up");
	fo->setGroup("standard-arrow");
	fo->setToolTip(tr("Very important!","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-arrow-2down.png"));
	fo->setName("2arrow-down");
	fo->setGroup("standard-arrow");
	fo->setToolTip(tr("Very unimportant!","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-thumb-up.png"));
	fo->setName("thumb-up");
	fo->setGroup("standard-thumb");
	fo->setToolTip(tr("I like this","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-thumb-down.png"));
	fo->setName("thumb-down");
	fo->setGroup("standard-thumb");
	fo->setToolTip(tr("I like this","Standardflag"));
	fo->setToolTip(tr("I do not like this","Standardflag"));
	standardFlagsDefault->addFlag (fo);	
	
	fo->load(QPixmap(flagsPath+"flag-rose.png"));
	fo->setName("rose");
	fo->setToolTip(tr("Rose","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-heart.png"));
	fo->setName("heart");
	fo->setToolTip(tr("I just love... ","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-present.png"));
	fo->setName("present");
	fo->setToolTip(tr("Surprise!","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-flash.png"));
	fo->setName("flash");
	fo->setToolTip(tr("Dangerous","Standardflag"));
	standardFlagsDefault->addFlag (fo);	
	
	fo->load(QPixmap(flagsPath+"flag-info.png"));
	// Original: xsldbg_output.png
	fo->setName("info");
	fo->setToolTip(tr("Info","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	fo->load(QPixmap(flagsPath+"flag-lifebelt.png"));
	// Original khelpcenter.png
	fo->setName("lifebelt");
	fo->setToolTip(tr("This will help","Standardflag"));
	standardFlagsDefault->addFlag (fo);	

	delete (fo);

	standardFlagsDefault->makeToolbar(this, "Standard Flags");
}

// Settings Actions
void Main::setupSettingsActions()
{
    Q3PopupMenu *menu = new Q3PopupMenu( this );
    menuBar()->insertItem( tr( "&Settings" ), menu );

	QAction *a;


    a = new QAction( tr( "Set application to open pdf files"), this);
    a->setStatusTip ( tr( "Set application to open pdf files"));
    connect( a, SIGNAL( activated() ), this, SLOT( settingsPDF() ) );
    a->addTo( menu );

    a = new QAction( tr( "Set application to open external links"), this);
    a->setStatusTip( tr( "Set application to open external links"));
    connect( a, SIGNAL( activated() ), this, SLOT( settingsURL() ) );
    a->addTo( menu );

    menu->insertSeparator();
    a = new QAction( tr( "Edit branch after adding it" ), this );
    a->setStatusTip( tr( "Edit branch after adding it" ));
	a->setToggleAction(true);
	a->setOn ( settings.readBoolEntry ("/vym/mapeditor/editmode/autoedit",true) );
    a->addTo( menu );
	actionSettingsAutoedit=a;

    a= new QAction( tr( "Select branch after adding it" ), this );
    a->setStatusTip( tr( "Select branch after adding it" ));
	a->setToggleAction(true);
	a->setOn ( settings.readBoolEntry ("/vym/mapeditor/editmode/autoselect",false) );
    a->addTo( menu );
	actionSettingsAutoselectHeading=a;
	
    a= new QAction(tr( "Select existing heading" ), this);
    a->setStatusTip( tr( "Select heading before editing" ));
	a->setToggleAction(true);
	a->setOn ( settings.readBoolEntry ("/vym/mapeditor/editmode/autoselectexistingtext",true) );
    a->addTo( menu );
	actionSettingsAutoselectText=a;
	
    a= new QAction(tr( "pasting into new branch" ), this );
    a->setStatusTip( tr( "Pasting into new branch" ));
	a->setToggleAction(true);
	a->setOn ( settings.readBoolEntry ("/vym/mapeditor/editmode/newheadingisempty",true) );
    a->addTo( menu );
	actionSettingsPasteNewHeading=a;
	
    a= new QAction( tr( "Delete key" ), this);
    a->setStatusTip( tr( "Delete key for deleting branches" ));
	a->setToggleAction(true);
	a->setOn ( settings.readBoolEntry ("/vym/mapeditor/editmode/useDelKey",false) );
    a->addTo( menu );
    connect( a, SIGNAL( activated() ), this, SLOT( settingsToggleDelKey() ) );
	actionSettingsUseDelKey=a;

    a= new QAction( tr( "Exclusive flags" ), this);
    a->setStatusTip( tr( "Use exclusive flags in flag toolbars" ));
	a->setToggleAction(true);
	a->setOn ( settings.readBoolEntry ("/vym/mapeditor/editmode/useFlagGroups",true) );
    a->addTo( menu );
	actionSettingsUseFlagGroups=a;
	
    a= new QAction( tr( "Use hide flags" ), this);
    a->setStatusTip( tr( "Use hide flag during exports " ));
	a->setToggleAction(true);
	a->setOn ( settings.readBoolEntry ("/vym/export/useHideExport",true) );
    a->addTo( menu );
	actionSettingsUseHideExport=a;
}

// Test Actions
void Main::setupTestActions()
{
    Q3PopupMenu *menu = new Q3PopupMenu( this );
    menuBar()->insertItem( tr( "&Test" ), menu );

    QAction *a;
    a = new QAction( tr( "test flag" ), this);
    a->setStatusTip( tr( "Call test function" ));
    connect( a, SIGNAL( activated() ), this, SLOT( testFunction() ) );
    a->addTo( menu );
}

// Help Actions
void Main::setupHelpActions()
{
    Q3PopupMenu *menu = new Q3PopupMenu( this );
    menuBar()->insertItem( tr( "&Help" ), menu );

    QAction *a;
    a = new QAction(  tr( "Open VYM Documentation (pdf) " ), this );
    a->setStatusTip( tr( "Open VYM Documentation (pdf)" ));
    connect( a, SIGNAL( activated() ), this, SLOT( helpDoc() ) );
    a->addTo( menu );

    a = new QAction( tr( "About VYM" ), this);
    a->setStatusTip( tr( "About VYM")+" "__VYM);
    connect( a, SIGNAL( activated() ), this, SLOT( helpAbout() ) );
    a->addTo( menu );

    a = new QAction( tr( "About QT" ), this);
    a->setStatusTip( tr( "Information about QT toolkit" ));
    connect( a, SIGNAL( activated() ), this, SLOT( helpAboutQT() ) );
    a->addTo( menu );
}

// Context Menus
void Main::setupContextMenus()
{
	QAction*a;

	// Context Menu for branch or mapcenter
	branchContextMenu =new Q3PopupMenu (this);

		// Submenu "Add"
		branchAddContextMenu =new Q3PopupMenu (this);
		actionEditPaste->addTo ( branchAddContextMenu );
		actionEditAddBranch->addTo ( branchAddContextMenu );
		actionEditAddBranchHere->addTo ( branchAddContextMenu );
		actionEditAddBranchAbove->addTo ( branchAddContextMenu );
		actionEditAddBranchBelow->addTo ( branchAddContextMenu );
		branchAddContextMenu->insertSeparator();	
		actionEditLoadImage->addTo( branchAddContextMenu );
		branchAddContextMenu->insertSeparator();	
		actionEditImportAdd->addTo ( branchAddContextMenu );
		actionEditImportReplace->addTo ( branchAddContextMenu );

		// Submenu "Remove"
		branchRemoveContextMenu =new Q3PopupMenu (this);
		actionEditCut->addTo ( branchRemoveContextMenu );
		actionEditDelete->addTo ( branchRemoveContextMenu );
		actionEditRemoveBranchKeepChilds->addTo( branchRemoveContextMenu );
		actionEditRemoveChilds->addTo( branchRemoveContextMenu );
		
	branchContextMenu->insertItem (tr("Add"),branchAddContextMenu);	
	branchContextMenu->insertItem (tr("Remove"),branchRemoveContextMenu);	

	actionEditSaveBranch->addTo( branchContextMenu );

	branchContextMenu->insertSeparator();	
	actionEditOpenURL->addTo ( branchContextMenu );
	actionEditOpenURLTab->addTo ( branchContextMenu );
	actionEditURL->addTo ( branchContextMenu );
	actionEditHeading2URL->addTo ( branchContextMenu );
	actionEditBugzilla2URL->addTo( branchContextMenu );
    if (settings.readBoolEntry( "/vym/mainwindow/showTestMenu",false)) 
	{
		actionEditFATE2URL->addTo( branchContextMenu );
	}	
	branchContextMenu->insertSeparator();	
	actionEditOpenVymLink->addTo ( branchContextMenu );
	actionEditVymLink->addTo ( branchContextMenu );
	actionEditDeleteVymLink->addTo ( branchContextMenu );
	
	branchContextMenu->insertSeparator();	
	actionGroupFormatFrameTypes->addTo( branchContextMenu );

	branchContextMenu->insertSeparator();	
	actionFormatIncludeImagesVer->addTo( branchContextMenu );
	actionFormatIncludeImagesHor->addTo( branchContextMenu );
	actionFormatHideLinkUnselected->addTo( branchContextMenu );

	// Context Menu for links in a branch menu
	// This will be populated "on demand" in MapEditor::updateActions
	branchContextMenu->insertSeparator();	
	branchLinksContextMenu =new Q3PopupMenu (this);
	branchLinksContextMenuDup =new Q3PopupMenu (this);
	branchContextMenu->insertItem (tr("Edit XLink"),branchLinksContextMenuDup);
	connect( branchLinksContextMenuDup, SIGNAL( activated(int) ), this, SLOT( editEditXLink(int ) ) );

	branchContextMenu->insertItem (tr("Goto XLink"),branchLinksContextMenu);
	connect( branchLinksContextMenu, SIGNAL( activated(int) ), this, SLOT( editFollowXLink(int ) ) );
	
	// Context menu for floatimage
	floatimageContextMenu =new Q3PopupMenu (this);
	saveImageFormatMenu=new Q3PopupMenu (this);
	exportImageFormatMenu=new Q3PopupMenu (this);

/* FIXME strange error
    foreach (QByteArray format, QImageWriter::supportedImageFormats()) 
	{
        QString text = tr("%1...").arg(QString(format).toUpper());
		saveImageFormatMenu->insertItem( text );
		exportImageFormatMenu->insertItem( text );
    }
*/

/* FIXME  not needed any longer
	Q3StrList fmt = QImageWriter::supportedImageFormats();
	for (const char* f = fmt.first(); f; f = fmt.next()) 
	{
		saveImageFormatMenu->insertItem( f );
		exportImageFormatMenu->insertItem( f );
	}	
*/	
	connect( saveImageFormatMenu, SIGNAL( activated(int) ), this, SLOT( editSaveImage(int ) ) );
	connect( exportImageFormatMenu, SIGNAL( activated(int) ), this, SLOT( fileExportImage(int ) ) );
	floatimageContextMenu->insertItem( tr("Save image"),saveImageFormatMenu );
	
	floatimageContextMenu->insertSeparator();	
	actionEditCopy->addTo( floatimageContextMenu );
	actionEditCut->addTo( floatimageContextMenu );

	floatimageContextMenu->insertSeparator();	
	actionFormatHideLinkUnselected->addTo( floatimageContextMenu );

	exportMenu->insertItem ( tr("Export as")+" Image",exportImageFormatMenu);

	a = new QAction( "Open Office"+QString("..."), this);
	a->setStatusTip( tr( "Export in Open Document Format used e.g. in Open Office " ));
	connect( a, SIGNAL( activated() ), this, SLOT( fileExportOOPresentation() ) );
	a->addTo (exportMenu);

	a = new QAction(  "Webpage (XHTML)...",this );
	a->setShortcut (Qt::ALT + Qt::Key_X);
	a->setStatusTip ( tr( "Export as")+" webpage (XHTML)");
    connect( a, SIGNAL( activated() ), this, SLOT( fileExportXHTML() ) );
    a->addTo( exportMenu );

    a = new QAction( "Text (ASCII)...", this);
	a->setStatusTip ( tr( "Export as")+" ASCII"+" "+tr("(still experimental)" ));
    connect( a, SIGNAL( activated() ), this, SLOT( fileExportASCII() ) );
	a->addTo( exportMenu );

	a = new QAction( tr("KDE Bookmarks"), this);
	a->setStatusTip( tr( "Export as")+" "+tr("KDE Bookmarks" ));
	connect( a, SIGNAL( activated() ), this, SLOT( fileExportKDEBookmarks() ) );
	a->addTo (exportMenu);

    a = new QAction( "Taskjuggler...", this );
    a->setStatusTip( tr( "Export as")+" Taskjuggler"+" "+tr("(still experimental)" ));
    connect( a, SIGNAL( activated() ), this, SLOT( fileExportTaskjuggler() ) );
	a->addTo( exportMenu );

    a = new QAction( "LaTeX...", this);
    a->setStatusTip( tr( "Export as")+" LaTeX"+" "+tr("(still experimental)" ));
    connect( a, SIGNAL( activated() ), this, SLOT( fileExportLaTeX() ) );
	a->addTo( exportMenu );

	a = new QAction( "XML..." , this );
	a->setStatusTip (tr( "Export as")+" XML");
    connect( a, SIGNAL( activated() ), this, SLOT( fileExportXML() ) );
    a->addTo( exportMenu );
	
	// Context menu for canvas
	canvasContextMenu =new Q3PopupMenu (this);
	actionEditMapInfo->addTo( canvasContextMenu );
	canvasContextMenu->insertSeparator();	
	actionGroupFormatLinkStyles->addTo( canvasContextMenu );
	canvasContextMenu->insertSeparator();	
	actionFormatLinkColorHint->addTo( canvasContextMenu );
	actionFormatLinkColor->addTo( canvasContextMenu );
	actionFormatBackColor->addTo( canvasContextMenu );

	// Menu for last opened files
	// Read settings initially
	QString s;
	int j=settings.readNumEntry( "/vym/lastMaps/number",0);
	for (int i=1;i<=j;i++)
	{
		s=settings.readEntry(QString("/vym/lastMaps/map-%1").arg(i),"");
		if (!s.isEmpty() && j<=maxLastMaps) 
			lastMaps.append(s);
	}
	setupLastMapsMenu();
	connect( lastMapsMenu, SIGNAL( activated(int) ), this, SLOT( fileLoadLast(int ) ) );
}

void Main::setupLastMapsMenu()
{
	// Remove double entries
	QStringList::Iterator it=lastMaps.begin();
	QStringList::Iterator jt;
	while (it!=lastMaps.end() )
	{
		jt=it;
		++jt;
		while (jt!=lastMaps.end() )
		{
			if (*it == *jt)		
				jt=lastMaps.remove(jt);
			else	
				jt++;
		}
		it++;
	}	

	// Limit length of list to maxLastMaps
	while ((int)(lastMaps.count()) > maxLastMaps) lastMaps.pop_back();
	
	// build Menu from lastMaps string list
	lastMapsMenu->clear();
	for (it = lastMaps.begin(); it != lastMaps.end(); ++it ) 
		lastMapsMenu->insertItem (*it );
		
}

void Main::hideEvent (QHideEvent * )
{
	if (!textEditor->isMinimized() ) textEditor->hide();
}

void Main::showEvent (QShowEvent * )
{
	if (textEditor->showWithMain()) textEditor->showNormal();
}

bool Main::reallyWriteDirectory(const QString &dir)
{
	QStringList eList = QDir(dir).entryList();
	if (eList.first() ==".")  eList.pop_front();	// remove "."
	if (eList.first() =="..") eList.pop_front();	// remove "."
	if (!eList.isEmpty())
	{
		QMessageBox mb( __VYM,
			tr("The directory %1 is not empty.\nDo you risk to overwrite its contents?").arg(dir),
		QMessageBox::Warning,
		QMessageBox::Yes ,
		QMessageBox::Cancel | QMessageBox::Default,
		QMessageBox::QMessageBox::NoButton );

		mb.setButtonText( QMessageBox::Yes, tr("Overwrite") );
		mb.setButtonText( QMessageBox::No, tr("Cancel"));
		switch( mb.exec() ) 
		{
			case QMessageBox::Yes:
				// save 
				return true;
			case QMessageBox::Cancel:
				// do nothing
				return false;
		}
	}
	return true;
}

QString Main::browseDirectory (const QString &caption)
{
	Q3FileDialog fd(this,caption);
	fd.setMode (Q3FileDialog::DirectoryOnly);
	fd.setCaption(__VYM " - "+caption);
	fd.show();
	
	if ( fd.exec() == QDialog::Accepted )
		return fd.selectedFile();
	else
		return "";
}

MapEditor* Main::currentMapEditor() const
{
    if ( tabWidget->currentPage() &&
	 tabWidget->currentPage()->inherits( "MapEditor" ) )
		return (MapEditor*)tabWidget->currentPage();
    return NULL;	
}

//TODO not used now, maybe use this for overview window later
void Main::newView() 
{
    // Open a new view... have it delete when closed.
    Main *m = new Main(0, 0, Qt::WDestructiveClose);
    qApp->setMainWidget(m);
    m->show();
    qApp->setMainWidget(0);
}

void Main::editorChanged(QWidget *)
{
	// Unselect all possibly selected objects
	// (Important to update note editor)
	int i;
	MapEditor *me;
	for (i=0;i<=tabWidget->count() -1;i++)
	{
		
		me=(MapEditor*)tabWidget->page(i);
		me->unselect();
	}	
	currentMapEditor()->reselect();

	// Update actions to in menus and toolbars according to editor
	currentMapEditor()->updateActions();
}

void Main::fileNew()
{
	QString fn="unnamed";
	MapEditor* medit = new MapEditor ( NULL,true);
	tabWidget->addTab (medit,fn);
	tabWidget->showPage(medit);
	medit->viewport()->setFocus();
	// For the very first map we do not have flagrows yet...
	medit->select("mc:");
}

ErrorCode Main::fileLoad(QString fn, const LoadMode &lmode)
{
	ErrorCode err=success;
	
	// fn is usually the archive, mapfile the file after uncompressing
	QString mapfile;

	// Make fn absolute (needed for unzip)
	fn=QDir (fn).absPath();

	MapEditor *me;

	if (lmode==NewMap)
	{
		// Check, if map is already loaded
		int i=0;
		while (i<=tabWidget->count() -1)
		{
			me=(MapEditor*)tabWidget->page(i);
			if (me->getFilePath() == fn)
			{
				// Already there, ask for confirmation
				QMessageBox mb( __VYM,
					tr("The map %1\nis already opened."
					"Opening the same map in multiple editors may lead \n"
					"to confusion when finishing working with vym."
					"Do you want to").arg(fn),
					QMessageBox::Warning,
					QMessageBox::Yes | QMessageBox::Default,
					QMessageBox::Cancel | QMessageBox::Escape,
					QMessageBox::NoButton);
				mb.setButtonText( QMessageBox::Yes, tr("Open anyway") );
				mb.setButtonText( QMessageBox::Cancel, tr("Cancel"));
				switch( mb.exec() ) 
				{
					case QMessageBox::Yes:
						// load anyway
						i=tabWidget->count();
						break;
					case QMessageBox::Cancel:
						// do nothing
						return aborted;
						break;
				}
			}
			i++;
		}
	}


	// Try to load map
    if ( !fn.isEmpty() )
	{
		me = currentMapEditor();
		int tabIndex=tabWidget->currentPageIndex();
		// Check first, if mapeditor exists
		// If it is not default AND we want a new map, 
		// create a new mapeditor in a new tab
		if ( lmode==NewMap && (!me || !me->isDefault() ) )
		{
			me = new MapEditor (tabWidget,true);
			tabWidget->addTab (me,fn);
			tabIndex=tabWidget->indexOf (me);
			tabWidget->setCurrentPage (tabIndex);
		}
		
		// Check, if file exists (important for creating new files
		// from command line
		if (!QFile(fn).exists() )
		{
			QMessageBox mb( __VYM,
				tr("This map does not exist:\n  %1\nDo you want to create a new one?").arg(fn),
				QMessageBox::Question,
				QMessageBox::Yes ,
				QMessageBox::Cancel | QMessageBox::Default,
				QMessageBox::NoButton );

			mb.setButtonText( QMessageBox::Yes, tr("Create"));
			mb.setButtonText( QMessageBox::No, tr("Cancel"));
			switch( mb.exec() ) 
			{
				case QMessageBox::Yes:
					// Create new map
					currentMapEditor()->setFilePath(fn);
					tabWidget->setTabLabel (currentMapEditor(),
						currentMapEditor()->getFileName() );
					statusBar()->message( "Created " + fn , statusbarTime );
					return success;
						
				case QMessageBox::Cancel:
					// don't create new map
					statusBar()->message( "Loading " + fn + " failed!", statusbarTime );
					fileCloseMap();
					return aborted;
			}
		}	


		//tabWidget->currentPage() won't be NULL here, because of above...
		tabWidget->showPage(me);
		me->viewport()->setFocus();

		// Create temporary directory for packing
		char tmpdir1[]="/tmp/vym-XXXXXX";	
		QString tmpMapDir=mkdtemp(tmpdir1);

		// Try to unzip file
		err=unzipDir (tmpMapDir,fn);
		if (err==nozip)
		{
			mapfile=fn;
			me->setZipped(false);
		} else
		{
			me->setZipped(true);
			
			// Look for mapname.xml
			mapfile= fn.left(fn.findRev(".",-1,true));
			mapfile=mapfile.section( '/', -1 );
			QFile file( tmpMapDir + "/" + mapfile + ".xml");
			if (!file.exists() )
			{
				// mapname.xml does not exist, well, 
				// maybe some renamed the mapname.vym file...
				// Try to find any .xml in the toplevel 
				// directory of the .vym file
				QStringList flist=QDir (tmpMapDir).entryList("*.xml");
				if (flist.count()==1) 
				{
					// Only one entry, take this one
					mapfile=tmpMapDir + "/"+flist.first();
				} else
				{
					for ( QStringList::Iterator it = flist.begin(); it != flist.end(); ++it ) 
						*it=tmpMapDir + "/" + *it;
					// TODO Multiple entries, load all (but only the first one into this ME)
					//mainWindow->fileLoadFromTmp (flist);
					//returnCode=1;	// Silently forget this attempt to load
					qWarning ("MainWindow::load (fn)  multimap found...");
				}	
					
				if (flist.isEmpty() )
				{
					QMessageBox::critical( 0, tr( "Critical Load Error" ),
							   tr("Couldn't find a map (*.xml) in .vym archive.\n"));
					err=aborted;				   
				}	
			} //file doesn't exist	
			else
				mapfile=file.name();
		}

		if (err!=aborted)
		{
			// Save existing filename in case  we import
			QString fn_org=me->getFilePath();

			// Finally load map into mapEditor
			me->setFilePath (mapfile,fn);
			err=me->load(mapfile,lmode);

			// Restore old (maybe empty) filepath, if this is an import
			if (lmode!=NewMap)
				me->setFilePath (fn_org);
		}	

		// Finally check for errors and go home
		if (err==aborted) 
		{
			if (lmode==NewMap) fileCloseMap();
			statusBar()->message( "Could not load " + fn, statusbarTime );
		} else 
		{
			if (lmode==NewMap)
			{
				me->setFilePath (fn);
				tabWidget->changeTab(tabWidget->page(tabIndex), me->getFileName());
				if (fn.left(9)!="/tmp/vym-")
				{
					// Only append to lastMaps if not loaded from a tmpDir
					// e.g. imported bookmarks are in a tmpDir
					lastMaps.prepend(me->getFilePath() );
					setupLastMapsMenu();
				}
				actionFilePrint->setEnabled (true);
			}	
			statusBar()->message( "Loaded " + fn, statusbarTime );
		}	

		// Delete tmpDir
		removeDir (QDir(tmpMapDir));
	}
	return err;
}


void Main::fileLoad(const LoadMode &lmode)
{
	Q3FileDialog *fd=new Q3FileDialog( this);
	if (!lastFileDir.isEmpty()) 
		fd->setDir (lastFileDir);
	fd->setMode (Q3FileDialog::ExistingFiles);
	fd->addFilter ("XML (*.xml)");
	fd->addFilter ("VYM map (*.vym *.vyp)");
	switch (lmode)
	{
		case NewMap:
			fd->setCaption(tr("Load vym map"));
			break;
		case ImportAdd:
			fd->setCaption(tr("Import: Add vym map to selection"));
			break;
		case ImportReplace:
			fd->setCaption(tr("Import: Replace selection with vym map"));
			break;
	}
	fd->show();

	QString fn;
	if ( fd->exec() == QDialog::Accepted )
	{
		lastFileDir=fd->dirPath();
	    QStringList flist = fd->selectedFiles();
		QStringList::Iterator it = flist.begin();
		while( it != flist.end() ) 
		{
			fn = *it;
			fileLoad(*it, lmode);				   
			++it;
		}
	}
	delete (fd);
}

void Main::fileLoad()
{
	fileLoad (NewMap);
}

void Main::fileLoadLast(int i)
{
	fileLoad(QString(*lastMaps.at(lastMapsMenu->indexOf (i)) ),NewMap);			   
}

void Main::fileSave(const SaveMode &savemode)
{
	// tmp dir for zipping 
	QString tmpMapDir;
	
	// Error codes
	ErrorCode err=success;
	
	QString safeFilePath;

	bool saveZipped=currentMapEditor()->saveZipped();

	MapEditor *	me=currentMapEditor();
	if (me)
	{
		QString fn=me->getFilePath();
		// filename=unnamed, filepath="" in constructor...
		if ( !fn.isEmpty() ) 
		{	
			// We have a filepath, go on saving			
			// First remove existing file, we 
			// don't want to add to old zip archives
			QFile f(fn);
			if (f.exists() ) 
				if (!f.remove())
					QMessageBox::warning( 0, tr( "Save Error" ),
						fn+   tr("\ncould not be removed before saving"));

			// Look, if we should zip the data:
			if (!saveZipped)
			{
				QMessageBox mb( __VYM,
					tr("The map %1\ndid not use the compressed "
					"vym file format.\nWriting it uncompressed will also write images \n"
					"and flags and thus may overwrite files in the "
					"given directory\n\nDo you want to write the map").arg(fn),
					QMessageBox::Warning,
					QMessageBox::Yes | QMessageBox::Default,
					QMessageBox::No ,
					QMessageBox::Cancel | QMessageBox::Escape);
				mb.setButtonText( QMessageBox::Yes, tr("compressed (vym default)") );
				mb.setButtonText( QMessageBox::No, tr("uncompressed") );
				mb.setButtonText( QMessageBox::Cancel, tr("Cancel"));
				switch( mb.exec() ) 
				{
					case QMessageBox::Yes:
						// save compressed (default file format)
						saveZipped=true;
						break;
					case QMessageBox::No:
						// save uncompressed
						saveZipped=false;
						break;
					case QMessageBox::Cancel:
						// do nothing
						return;
						break;
				}
			}

			if (saveZipped)
			{
				char tmpdir1[]="/tmp/vym-XXXXXX";	
				tmpMapDir=mkdtemp(tmpdir1);
			
				safeFilePath=me->getFilePath();
				me->setFilePath (tmpMapDir+"/"+
					me->getMapName()+ ".xml",
					safeFilePath);
				me->save (savemode);
				me->setFilePath (safeFilePath);
				
				zipDir (tmpMapDir,fn);
			} // save zipped
			else
			{
				// Save unzipped. 
				safeFilePath=me->getFilePath();
				me->setFilePath (fn, safeFilePath);
				me->save (savemode);
				me->setFilePath (safeFilePath);
			} // save zipped 	
		} // filepath available
		else
		{
			// We have  no filepath yet,
			// call fileSaveAs() now, this will call fileSave() 
			// again.
			fileSaveAs(savemode);
		}
    }

	if (saveZipped && !tmpMapDir.isEmpty())
		// Delete tmpDir
		removeDir (QDir(tmpMapDir));

	if (err==success)
	{
		statusBar()->message( 
			tr("Saved  %1").arg(me->getFilePath()), 
			statusbarTime );
		lastMaps.prepend(me->getFilePath() );
		setupLastMapsMenu();
	} else		
		statusBar()->message( 
			tr("Couldn't save ").arg(me->getFilePath()), 
			statusbarTime );
}

void Main::fileSave()
{
	fileSave (CompleteMap);
}

void Main::fileSaveAs(const SaveMode& savemode)
{
	QString fn;

	if (currentMapEditor())
	{
		if (savemode==CompleteMap)
			fn = Q3FileDialog::getSaveFileName( QString::null, "VYM map (*.vym)", this );
		else		
			fn = Q3FileDialog::getSaveFileName( QString::null, "VYM part of map (*.vyp)", this );
		if ( !fn.isEmpty() ) 
		{
			// Check for existing file
			if (QFile (fn).exists())
			{
				QMessageBox mb( __VYM,
					tr("The file %1\nexists already. Do you want to").arg(fn),
					QMessageBox::Warning,
					QMessageBox::Yes | QMessageBox::Default,
					QMessageBox::Cancel | QMessageBox::Escape,
					QMessageBox::NoButton);
				mb.setButtonText( QMessageBox::Yes, tr("Overwrite") );
				mb.setButtonText( QMessageBox::Cancel, tr("Cancel"));
				switch( mb.exec() ) 
				{
					case QMessageBox::Yes:
						// save 
						break;
					case QMessageBox::Cancel:
						// do nothing
						return;
						break;
				}
			} else
			{
				// New file, add extension to filename, if missing
				// This is always .vym or .vyp, depending on savemode
				if (savemode==CompleteMap)
				{
					if (!fn.contains (".vym") && !fn.contains (".xml"))
						fn +=".vym";
				} else		
				{
					if (!fn.contains (".vyp") && !fn.contains (".xml"))
						fn +=".vyp";
				}
			}
	



			// Save now
			currentMapEditor()->setFilePath(fn);
			fileSave(savemode);

			// Set name of tab
			if (savemode==CompleteMap)
				tabWidget->setTabLabel (currentMapEditor(),
					currentMapEditor()->getFileName() );
			return;
		} 
	}
}

void Main::fileSaveAs()
{
	fileSaveAs (CompleteMap);
}

void Main::fileImportKDEBookmarks()
{
	ImportKDEBookmarks im;
	im.transform();
	if (success==fileLoad (im.getTransformedFile(),NewMap) && currentMapEditor() )
		currentMapEditor()->setFilePath ("");
}

void Main::fileImportFirefoxBookmarks()
{
	Q3FileDialog *fd=new Q3FileDialog( this);
	fd->setDir (vymBaseDir.homeDirPath()+"/.mozilla/firefox");
	fd->setMode (Q3FileDialog::ExistingFiles);
	fd->addFilter ("Firefox "+tr("Bookmarks")+" (*.html)");
	fd->setCaption(tr("Import")+" "+"Firefox "+tr("Bookmarks"));
	fd->show();

	if ( fd->exec() == QDialog::Accepted )
	{
		ImportFirefoxBookmarks im;
	    QStringList flist = fd->selectedFiles();
		QStringList::Iterator it = flist.begin();
		while( it != flist.end() ) 
		{
			im.setFile (*it);
			if (im.transform() && 
				success==fileLoad (im.getTransformedFile(),NewMap) && 
				currentMapEditor() )
				currentMapEditor()->setFilePath ("");
			++it;
		}
	}
	delete (fd);
}

void Main::fileImportMM()
{
	ImportMM im;

	Q3FileDialog *fd=new Q3FileDialog( this);
	if (!lastFileDir.isEmpty()) 
		fd->setDir (lastFileDir);
	fd->setMode (Q3FileDialog::ExistingFiles);
	fd->addFilter ("Mind Manager (*.mmap)");
	fd->setCaption(tr("Import")+" "+"Mind Manager");
	fd->show();

	if ( fd->exec() == QDialog::Accepted )
	{
		lastFileDir=fd->dirPath();
	    QStringList flist = fd->selectedFiles();
		QStringList::Iterator it = flist.begin();
		while( it != flist.end() ) 
		{
			im.setFile (*it);
			if (im.transform() && 
				success==fileLoad (im.getTransformedFile(),NewMap) && 
				currentMapEditor() )
				currentMapEditor()->setFilePath ("");

			++it;
		}
	}
	delete (fd);

}

void Main::fileImportDir()
{
	if (currentMapEditor())
		currentMapEditor()->importDir();	
}

void Main::fileExportXML()
{
	if (currentMapEditor())
	{
		QString dir=browseDirectory(tr("Export XML to directory"));
		if (dir !="" && reallyWriteDirectory(dir) )
			currentMapEditor()->exportXML(dir);
	}	
}


void Main::fileExportXHTML()
{
	MapEditor *me=currentMapEditor();
	QString dir;
	if (me)
	{
		ExportXHTMLDialog dia(this);
		dia.setFilePath (me->getFilePath() );
		dia.setMapName (me->getMapName() );
		dia.readSettings();
		
		if (dia.exec()==QDialog::Accepted)
		{
			QString dir=dia.getDir();
			// Check, if warnings should be used before overwriting
			// the output directory
			bool ok;
			if (dia.warnings()) 
				ok=reallyWriteDirectory(dir);
			else
				ok=true;

			if (ok)
			{
				me->exportXML (dia.getDir() );
				dia.doExport(me->getMapName() );
				if (dia.hasChanged())
					me->setChanged();
			}	
		}
	}	
}

void Main::fileExportImage(int item)
{
	MapEditor *me=currentMapEditor();
	if (me)
	{
		QString fn = Q3FileDialog::getSaveFileName( QString::null, "Image  (*.bmp *.jpg *.pbm *.pgm *.png *.ppm *xbm *.xpm)",
							   this );
		if ( !fn.isEmpty() ) 
			me->exportImage(fn,item);
		else 
			statusBar()->message( tr("Couldn't save %1").arg(fn), statusbarTime );
		
	}
}

void Main::fileExportASCII()
{
	MapEditor *me=currentMapEditor();
	if (me)
	{
		ExportASCII ex;
		ex.setMapCenter(me->getMapCenter());
		ex.addFilter ("TXT (*.txt)");
		ex.setCaption(__VYM " -" +tr("Export as ASCII")+" "+tr("(still experimental)"));
		if (ex.execDialog() ) 
		{
			me->setExportMode(true);
			ex.doExport();
			me->setExportMode(false);
		}
	}
}

void Main::fileExportLaTeX()
{
	MapEditor *me=currentMapEditor();
	if (me)
	{
		ExportLaTeX ex;
		ex.setMapCenter(me->getMapCenter());
		ex.addFilter ("Tex (*.tex)");
		ex.setCaption(__VYM " -" +tr("Export as LaTeX")+" "+tr("(still experimental)"));
		if (ex.execDialog() ) 
		{
			me->setExportMode(true);
			ex.doExport();
			me->setExportMode(false);
		}
	}
}

void Main::fileExportKDEBookmarks()
{
	ExportKDEBookmarks ex;
	MapEditor *me=currentMapEditor();
	if (me)
	{
		ex.setMapCenter (me->getMapCenter() );
		ex.doExport();
	}	
}

void Main::fileExportTaskjuggler()
{
	ExportTaskjuggler ex;
	MapEditor *me=currentMapEditor();
	if (me)
	{
		ex.setMapCenter (me->getMapCenter() );
		ex.setCaption ( __VYM " - "+tr("Export to")+" Taskjuggler"+tr("(still experimental)"));
		ex.addFilter ("Taskjuggler (*.tjp)");
		if (ex.execDialog() ) 
		{
			me->setExportMode(true);
			ex.doExport();
			me->setExportMode(false);
		}
	}	
}

void Main::fileExportOOPresentation()
{
	ExportOOFileDialog *fd=new ExportOOFileDialog( this,__VYM " - "+tr("Export to")+" Open Office");
	// FIXME add extra info in dialog
	//ImagePreview *p =new ImagePreview (fd);
	//fd->setContentsPreviewEnabled( TRUE );
	//fd->setContentsPreview( p, p );
	//fd->setPreviewMode( QFileDialog::Contents );
	fd->setCaption(__VYM " - " +tr("Export to")+" Open Office");
	//fd->setDir (lastImageDir);
	fd->show();

	if ( fd->exec() == QDialog::Accepted )
	{
		QString fn=fd->selectedFile();
		//lastImageDir=fn.left(fn.findRev ("/"));
		if (currentMapEditor())
			currentMapEditor()->exportOOPresentation(fn,fd->selectedConfig());	
	}
}

void Main::fileCloseMap()
{
	if (currentMapEditor())
	{
		if (currentMapEditor()->hasChanged())
		{
			QMessageBox mb( __VYM,
				tr("The map %1 has been modified but not saved yet. Do you want to").arg(currentMapEditor()->getFileName()),
				QMessageBox::Warning,
				QMessageBox::Yes | QMessageBox::Default,
				QMessageBox::No,
				QMessageBox::Cancel | QMessageBox::Escape );
			mb.setButtonText( QMessageBox::Yes, tr("Save modified map before closing it") );
			mb.setButtonText( QMessageBox::No, tr("Discard changes"));
			switch( mb.exec() ) 
			{
				case QMessageBox::Yes:
					// save and close
					fileSave(CompleteMap);
					break;
				case QMessageBox::No:
				// close  without saving
					break;
				case QMessageBox::Cancel:
					// do nothing
				return;
			}
		} 
		currentMapEditor()->closeMap();
		tabWidget->removePage(currentMapEditor());
		if (tabWidget->count()==0)
			actionFilePrint->setEnabled (false);
	}	
}

void Main::filePrint()
{
	if (currentMapEditor())
		currentMapEditor()->print();
}

void Main::fileExitVYM()
{
	// Check if one or more editors have changed
	MapEditor *me;
	int i;
	for (i=0;i<=tabWidget->count() -1;i++)
	{
		
		me=(MapEditor*)tabWidget->page(i);

		// If something changed, ask what to do
		if (me->isUnsaved())
		{
			tabWidget->setCurrentPage(i);
			QMessageBox mb( __VYM,
				tr("This map is not saved yet. Do you want to"),
				QMessageBox::Warning,
				QMessageBox::Yes | QMessageBox::Default,
				QMessageBox::No,
				QMessageBox::Cancel | QMessageBox::Escape );
			mb.setButtonText( QMessageBox::Yes, tr("Save map") );
			mb.setButtonText( QMessageBox::No, tr("Discard changes") );
			mb.show();
			mb.setActiveWindow();
			switch( mb.exec() ) {
				case QMessageBox::Yes:
					// save (the changed editors) and exit
					fileSave(CompleteMap);
					break;
				case QMessageBox::No:
					// exit without saving
					break;
				case QMessageBox::Cancel:
					// don't save and don't exit
				return;
			}
		}
	} // loop over all MEs	
    qApp->quit();
}

void Main::editUndo()
{
	if (currentMapEditor())
		currentMapEditor()->undo();
}

void Main::editRedo()	   
{
	if (currentMapEditor())
		currentMapEditor()->redo();
}

void Main::editCopy()
{
	if (currentMapEditor())
		currentMapEditor()->copy();
}

void Main::editPaste()
{
	if (currentMapEditor())
		currentMapEditor()->paste();
}

void Main::editCut()
{
	if (currentMapEditor())
		currentMapEditor()->cut();
}

void Main::editOpenFindWindow()
{
	findWindow->popup();
	findWindow->raise();
	findWindow->setActiveWindow();
}

void Main::editFind(QString s)
{
	bool cs=false;
	BranchObj *bo=currentMapEditor()->findText(s, cs);
	if (bo)
	{	
		statusBar()->message( "Found: " + bo->getHeading(), statusbarTime );
	} else
	{
		QMessageBox::information( findWindow, tr( "VYM -Information:" ),
							   tr("No matches found for \"%1\"").arg(s));
	}	
}

void Main::editFindChanged()
{	// Notify editor, to abort the current find process
	currentMapEditor()->findReset();
}

void Main::editOpenURL()
{
	// Open new browser
	if (currentMapEditor())
	{	
	    QString url=currentMapEditor()->getURL();
		if (url=="") return;
		QString browser=settings.readEntry("/vym/mainwindow/readerURL" );
		procBrowser = new Q3Process( this );

		procBrowser->addArgument( browser);
		procBrowser->addArgument( url);

		if ( !procBrowser->start() ) 
		{
			// try to set path to browser
			QMessageBox::warning(0, 
				tr("Warning"),
				tr("Couldn't find a viewer to open %1.\n").arg(url)+
				tr("Please use Settings->")+tr("Set application to open an URL"));
			settingsURL() ; 
		}	
	}	
}
void Main::editOpenURLTab()
{
	if (currentMapEditor())
	{	
	    QString url=currentMapEditor()->getURL();
		if (url=="") return;
		QString browser=settings.readEntry("/vym/mainwindow/readerURL" );
		if (procBrowser && procBrowser->isRunning())
		{
			if (browser.contains("konqueror"))
			{
				// Try to open new tab in existing konqueror started previously by vym
				Q3Process *p=new Q3Process (this);
				p->addArgument ("dcop");
				p->addArgument (QString("konqueror-%1").arg(procBrowser->processIdentifier()));
				p->addArgument ("konqueror-mainwindow#1");
				p->addArgument ("newTab");
				p->addArgument (url);
				if ( !p->start() ) 
					// try to set browser
					QMessageBox::warning(0, 
						tr("Warning"),
						tr("Couldn't start %1 to open a new tab in %2.").arg("dcop").arg("konqueror"));
				return;		
			} else if (browser.contains ("firefox") || browser.contains ("mozilla") )
			{
				// Try to open new tab in firefox
				Q3Process *p=new Q3Process (this);
				p->addArgument (browser);
				p->addArgument ("-remote");
				p->addArgument (QString("openurl(%1,new-tab)").arg(url));
				if ( !p->start() ) 
					// try to set browser
					QMessageBox::warning(0, 
						tr("Warning"),
						tr("Couldn't start %1 to open a new tab").arg(browser));
				return;		
			}
		}
		// Open new browser
		editOpenURL();
	}	
}

void Main::editURL()
{
	if (currentMapEditor())
	    currentMapEditor()->editURL();
}

void Main::editHeading2URL()
{
	if (currentMapEditor())
	    currentMapEditor()->editHeading2URL();
}

void Main::editBugzilla2URL()
{
	if (currentMapEditor())
	    currentMapEditor()->editBugzilla2URL();
}

void Main::editFATE2URL()
{
	if (currentMapEditor())
	    currentMapEditor()->editFATE2URL();
}

void Main::editOpenVymLink()
{
	// Get current path to map
	QString currentVymLink;
	if (currentMapEditor())
	{
		currentVymLink=currentMapEditor()->getVymLink();	
		// compare path with already loaded maps
		int index=-1;
		int i;
		MapEditor *me;
		for (i=0;i<=tabWidget->count() -1;i++)
		{
			me=(MapEditor*)tabWidget->page(i);
			if (currentVymLink==me->getFilePath() )
			{
				index=i;
				break;
			}
		}	
		if (index<0)
		// Load map
		{
			if (!QFile(currentVymLink).exists() )
				QMessageBox::critical( 0, tr( "Critical Error" ),
				   tr("Couldn't open map %1").arg(currentVymLink));
			else
			{
				fileLoad (currentVymLink, NewMap);
				tabWidget->setCurrentPage (tabWidget->count()-1);	
			}
		} else
			// Go to tab containing the map
			tabWidget->setCurrentPage (index);	
	}
}

void Main::editVymLink()
{
	if (currentMapEditor())
		currentMapEditor()->editVymLink();	
}

void Main::editDeleteVymLink()
{
	if (currentMapEditor())
		currentMapEditor()->deleteVymLink();	
}

void Main::editToggleHideExport()
{
	if (currentMapEditor())
		currentMapEditor()->toggleHideExport();	
}

void Main::editMapInfo()
{
	if (currentMapEditor())
		currentMapEditor()->editMapInfo();	
}

void Main::editMoveUp()
{
	if (currentMapEditor())
	    currentMapEditor()->moveBranchUp();
}

void Main::editMoveDown()
{
	if (currentMapEditor())
		currentMapEditor()->moveBranchDown();
}

void Main::editToggleScroll()
{
	if (currentMapEditor())
	{
		currentMapEditor()->toggleScroll();	
	}	
}

void Main::editUnScrollAll()
{
	if (currentMapEditor())
	{
		currentMapEditor()->unScrollAll();	
	}	
}

void Main::editHeading()
{
	if (currentMapEditor())
		currentMapEditor()->editHeading();
}

void Main::editNewBranch()
{
	if (currentMapEditor())
		currentMapEditor()->addNewBranch(0);
}

void Main::editNewBranchHere()
{
	if (currentMapEditor())
		currentMapEditor()->addNewBranchHere();
}

void Main::editNewBranchAbove()
{
	if (currentMapEditor())
		currentMapEditor()->addNewBranch(-1);
}

void Main::editNewBranchBelow()
{
	if (currentMapEditor())
		currentMapEditor()->addNewBranch(1);
}

void Main::editImportAdd()
{
	fileLoad (ImportAdd);
}

void Main::editImportReplace()
{
	fileLoad (ImportReplace);
}

void Main::editSaveBranch()
{
	fileSaveAs (PartOfMap);
}

void Main::editRemoveBranchKeepChilds()
{
	if (currentMapEditor())
		currentMapEditor()->removeBranchKeepChilds();
}

void Main::editRemoveChilds()
{
	if (currentMapEditor())
		currentMapEditor()->removeChilds();
}

void Main::editDeleteSelection()
{
	if (currentMapEditor() && actionSettingsUseDelKey->isOn())
		currentMapEditor()->deleteSelection();
}

void Main::editUpperBranch()
{
	if (currentMapEditor())
		currentMapEditor()->selectUpperBranch();
}

void Main::editLowerBranch()
{
	if (currentMapEditor())
		currentMapEditor()->selectLowerBranch();
}

void Main::editLeftBranch()
{
	if (currentMapEditor())
		currentMapEditor()->selectLeftBranch();
}

void Main::editRightBranch()
{
	if (currentMapEditor())
		currentMapEditor()->selectRightBranch();
}

void Main::editFirstBranch()
{
	if (currentMapEditor())
		currentMapEditor()->selectFirstBranch();
}

void Main::editLastBranch()
{
	if (currentMapEditor())
		currentMapEditor()->selectLastBranch();
}

void Main::editLoadImage()
{
	if (currentMapEditor())
		currentMapEditor()->loadFloatImage();
}

void Main::editSaveImage(int item)
{
	if (currentMapEditor())
		currentMapEditor()->saveFloatImage(item);
}

void Main::editFollowXLink(int item)
{
	if (currentMapEditor())
		currentMapEditor()->followXLink(branchLinksContextMenu->indexOf(item));
}

void Main::editEditXLink(int item)
{
	if (currentMapEditor())
		currentMapEditor()->editXLink(branchLinksContextMenuDup->indexOf(item));
}

void Main::formatSelectColor()
{
	if (currentMapEditor())
	{
		QColor col = QColorDialog::getColor( currentMapEditor()->color(), this );
		if ( !col.isValid() ) return;
		currentMapEditor()->setColor( col );
		colorChanged( col );
	}	
}

void Main::formatPickColor()
{
	if (currentMapEditor())
		colorChanged( currentMapEditor()->pickColor() );
}

void Main::colorChanged(QColor c)
{
    QPixmap pix( 16, 16 );
    pix.fill( c );
    actionFormatColor->setIconSet( pix );
}

void Main::formatColorItem()
{
	if (currentMapEditor())
		currentMapEditor()->colorItem();
}

void Main::formatColorBranch()
{
	if (currentMapEditor())
		currentMapEditor()->colorBranch();
}

void Main::formatLinkStyleLine()
{
	if (currentMapEditor())
		currentMapEditor()->setLinkStyle(StyleLine);
}

void Main::formatLinkStyleParabel()
{
	if (currentMapEditor())
		currentMapEditor()->setLinkStyle(StyleParabel);
}

void Main::formatLinkStylePolyLine()
{
	if (currentMapEditor())
		currentMapEditor()->setLinkStyle(StylePolyLine);
}

void Main::formatLinkStylePolyParabel()
{
	if (currentMapEditor())
		currentMapEditor()->setLinkStyle(StylePolyParabel);
}

void Main::formatSelectBackColor()
{
	if (currentMapEditor())
		currentMapEditor()->selectBackgroundColor();
}

void Main::formatSelectLinkColor()
{
	if (currentMapEditor())
		currentMapEditor()->selectLinkColor();
}

void Main::formatToggleLinkColorHint()
{
	currentMapEditor()->toggleLinkColorHint();
}

void Main::formatFrameNone()
{
	if (currentMapEditor())
		currentMapEditor()->setFrame(NoFrame);
}

void Main::formatFrameRectangle()
{
	if (currentMapEditor())
		currentMapEditor()->setFrame(Rectangle);
}

void Main::formatIncludeImagesVer()
{
	if (currentMapEditor())
		currentMapEditor()->setIncludeImagesVer(actionFormatIncludeImagesVer->isOn());
}

void Main::formatIncludeImagesHor()
{
	if (currentMapEditor())
		currentMapEditor()->setIncludeImagesHor(actionFormatIncludeImagesHor->isOn());
}

void Main::formatHideLinkUnselected()
{
	if (currentMapEditor())
		currentMapEditor()->setHideLinkUnselected(actionFormatHideLinkUnselected->isOn());
}

void Main::viewZoomReset()
{
	if (currentMapEditor())
	{
		QMatrix m;
		m.reset();
		currentMapEditor()->setWorldMatrix( m );
		currentMapEditor()->setViewCenter();
		currentMapEditor()->adjustCanvasSize();
	}	
}

void Main::viewZoomIn()
{
	if (currentMapEditor())
	{
		QMatrix m = currentMapEditor()->worldMatrix();
		m.scale( 1.25, 1.25 );
		currentMapEditor()->setWorldMatrix( m );
		currentMapEditor()->setViewCenter();
		currentMapEditor()->adjustCanvasSize();
	}	
}

void Main::viewZoomOut()
{
	if (currentMapEditor())
	{
		QMatrix m = currentMapEditor()->worldMatrix();
		m.scale( 0.8, 0.8 );
		currentMapEditor()->setWorldMatrix( m );
		currentMapEditor()->setViewCenter();
		currentMapEditor()->adjustCanvasSize();
	}	
}

void Main::modModeColor()
{
}

void Main::modModeLink()
{
}

bool Main::settingsPDF()
{
	// Default browser is set in constructor
	bool ok;
	QString text = QInputDialog::getText(
		"VYM", tr("Set application to open PDF files")+":", QLineEdit::Normal,
		settings.readEntry("/vym/mainwindow/readerPDF"), &ok, this );
	if (ok)
		settings.writeEntry ("/vym/mainwindow/readerPDF",text);
	return ok;
}


bool Main::settingsURL()
{
	// Default browser is set in constructor
	bool ok;
	QString text = QInputDialog::getText(
		"VYM", tr("Set application to open an URL")+":", QLineEdit::Normal,
		settings.readEntry("/vym/mainwindow/readerURL")
		, &ok, this );
	if (ok)
		settings.writeEntry ("/vym/mainwindow/readerURL",text);
	return ok;
}

void Main::settingsToggleDelKey()
{
	if (actionSettingsUseDelKey->isOn())
	{
		actionEditDelete->setAccel (QKeySequence (Qt::Key_Delete));
	} else
	{
		actionEditDelete->setAccel (QKeySequence (""));
	}
}

void Main::windowToggleNoteEditor()
{
	if (textEditor->showWithMain() )
		windowHideNoteEditor();
	else	
		windowShowNoteEditor();
}

void Main::windowToggleHistory()
{
	if (currentMapEditor())
		currentMapEditor()->toggleHistoryWindow();
}

void Main::updateNoteFlag()
{
	if (currentMapEditor())
		currentMapEditor()->updateNoteFlag();
}

void Main::windowShowNoteEditor()
{
	textEditor->setShowWithMain(true);
	textEditor->show();
	actionViewToggleNoteEditor->setOn (true);
}

void Main::windowHideNoteEditor()
{
	textEditor->setShowWithMain(false);
	textEditor->hide();
	actionViewToggleNoteEditor->setOn (false);
}

void Main::windowNextEditor()
{
	if (tabWidget->currentPageIndex() < tabWidget->count())
		tabWidget->setCurrentPage (tabWidget->currentPageIndex() +1);
}

void Main::windowPreviousEditor()
{
	if (tabWidget->currentPageIndex() >0)
		tabWidget->setCurrentPage (tabWidget->currentPageIndex() -1);
}

void Main::standardFlagChanged()
{
	currentMapEditor()->toggleStandardFlag(sender()->name());
}

void Main::testFunction()
{
	currentMapEditor()->testFunction();
}

void Main::helpDoc()
{
	QString docpath;
	#if defined(Q_OS_MACX)
		docpath="./vym.app/Contents/vym.pdf";
	#else
		// default path in SUSE LINUX
		docpath="/usr/share/doc/packages/vym/doc/vym.pdf";
	#endif

	if (!QFile (docpath).exists() )
	{
		// relative path for easy testing in tarball
		docpath="doc/vym.pdf";
		if (!QFile (docpath).exists() )
		{
			// relative path for testing while still writing vym.tex
			docpath="doc/tex/vym.pdf";
			if (!QFile (docpath).exists() )
			{
				// Try yet another one for Knoppix
				docpath="/usr/share/doc/packages/vym/vym.pdf";
				if (!QFile (docpath).exists() )
				{
					QMessageBox::critical(0, 
					tr("Critcal error"),
					tr("Couldn't find the documentation\n"
					"vym.pdf in various places."));
					return;
				}	
			}	
		}
	}
	
	Process *pdfProc = new Process();
	pdfProc->clearArguments();
	pdfProc->addArgument( settings.readEntry("/vym/mainwindow/readerPDF"));
	pdfProc->addArgument( docpath);

	if ( !pdfProc->start() ) 
	{
		// error handling
		QMessageBox::warning(0, 
			tr("Warning"),
			tr("Couldn't find a viewer to open %1.\n").arg("vym.pdf")+
			tr("Please use Settings->")+tr("Set application to open PDF files"));
		settingsPDF();	
		return;
	}
}


void Main::helpAbout()
{
	AboutDialog ad;
	ad.setName ("aboutwindow");
	ad.setMinimumSize(500,500);
	ad.resize (QSize (500,500));
	ad.exec();
}

void Main::helpAboutQT()
{
	QMessageBox::aboutQt( this, "Qt Application Example" );
}

